# remirepo spec file for php-pecl-skywalking-agent
#
# Copyright (c) 2022-2023 Remi Collet
# License: CC-BY-SA-4.0
# http://creativecommons.org/licenses/by-sa/4.0/
#
# Please, preserve the changelog entries
#

%if 0%{?scl:1}
%scl_package       php-pecl-skywalking-agent
%else
%global _root_bindir %{_bindir}
%endif

%global pecl_name  skywalking_agent
# not supported by upstream
%global with_zts   0
%global ini_name   40-%{pecl_name}.ini
%global sources    %{pecl_name}-%{version}
%global _configure ../%{sources}/configure

## BUILD: Fedora / EL-8 and EL-9 using --enablerepo=stream if needed
## EL-7 not supported (This crate only supports `libclang` 3.5 and later)

## TODO: issues
# no support for offline build
# => workaround with "cargo vendor"


Summary:        Apache SkyWalking PHP Agent
Name:           %{?scl_prefix}php-pecl-skywalking-agent
Version:        0.7.0
Release:        1%{?dist}%{!?nophptag:%(%{__php} -r 'echo ".".PHP_MAJOR_VERSION.".".PHP_MINOR_VERSION;')}
License:        Apache-2.0
URL:            https://pecl.php.net/package/%{pecl_name}

Source0:        https://pecl.php.net/get/%{sources}.tgz
# awfull hack, use a bundled rust registry
Source1:        makedeps.sh
Source2:        %{pecl_name}-deps-%{version}.tgz

%if 0%{?rhel} == 7 && 0%{?dtsversion} == 0
BuildRequires:  devtoolset-6-toolchain
%global dtsversion 6
%global dtsenable source /opt/rh/devtoolset-6/enable
%global dtsprefix devtoolset-6-
%endif
BuildRequires:  make
BuildRequires:  %{?dtsprefix}gcc
BuildRequires:  %{?dtsprefix}gcc-c++
BuildRequires:  %{?scl_prefix}php-devel >= 7.2
BuildRequires:  %{?scl_prefix}php-pear
BuildRequires:  cargo >= 1.65
BuildRequires:  rustfmt
BuildRequires:  protobuf-compiler
BuildRequires:  clang-devel
%if 0%{?fedora} >= 27 || 0%{?rhel} >= 9
BuildRequires:  rust-packaging
%endif

Requires:       %{?scl_prefix}php(zend-abi) = %{php_zend_api}
Requires:       %{?scl_prefix}php(api) = %{php_core_api}

Provides:       %{?scl_prefix}php-%{pecl_name}               = %{version}
Provides:       %{?scl_prefix}php-%{pecl_name}%{?_isa}       = %{version}
Provides:       %{?scl_prefix}php-pecl-%{pecl_name}          = %{version}
Provides:       %{?scl_prefix}php-pecl-%{pecl_name}%{?_isa}  = %{version}
Provides:       %{?scl_prefix}php-pecl(%{pecl_name})         = %{version}
Provides:       %{?scl_prefix}php-pecl(%{pecl_name})%{?_isa} = %{version}


%description
The PHP Agent for Apache SkyWalking, which provides the native tracing
abilities for PHP project.

Package built for PHP %(%{__php} -r 'echo PHP_MAJOR_VERSION.".".PHP_MINOR_VERSION;')%{?scl: as Software Collection (%{scl} by %{?scl_vendor}%{!?scl_vendor:rh})}.


%prep
%setup -q -c -a2

# Don't install/register tests
sed -e 's/role="test"/role="src"/' \
    %{?_licensedir:-e '/LICENSE/s/role="doc"/role="src"/' } \
    -i package.xml

# Create cargo configuration to use vendor directory
mkdir .cargo
cat << EOF | tee .cargo/config.toml
[build]
jobs = %(echo %{?_smp_mflags} | sed 's/\-j//')
rustc = "%{_root_bindir}/rustc"

[env]
CFLAGS = "%{build_cflags}"
CXXFLAGS = "%{build_cxxflags}"
LDFLAGS = "%{build_ldflags}"

[term]
verbose = true

[source.crates-io]
replace-with = "vendored-sources"

[source.vendored-sources]
directory = "$PWD/mycargo"
EOF

pushd %{sources}
: Check version
grep '^version' Cargo.toml
grep '^version = "%{version}"' Cargo.toml
popd

mkdir NTS
%if %{with_zts}
# Duplicate source tree for NTS / ZTS build
mkdir ZTS
%endif

# Create configuration file
cat > %{ini_name} << 'EOF'
; Enable %{pecl_name} extension module
extension=%{pecl_name}.so

; Configuration
;skywalking_agent.enable = 0
;skywalking_agent.skywalking_version = 8
;skywalking_agent.server_addr = 127.0.0.1:11800
;skywalking_agent.service_name = hello-skywalking
;skywalking_agent.worker_threads = 0
;skywalking_agent.log_level = OFF
;skywalking_agent.log_file = /tmp/skywalking-agent.log
;skywalking_agent.runtime_dir = /tmp/skywalking-agent
;skywalking_agent.authentication =
;skywalking_agent.enable_tls = 0
;skywalking_agent.ssl_trusted_ca_path =
;skywalking_agent.ssl_key_path =
;skywalking_agent.ssl_cert_chain_path =
;skywalking_agent.heartbeat_period = 30
;skywalking_agent.properties_report_period_factor = 10
;skywalking_agent.enable_zend_observer = 0
;skywalking_agent.reporter_type = grpc
;skywalking_agent.kafka_bootstrap_servers =
;skywalking_agent.kafka_producer_config = {}
EOF

: Required rust version
grep -h rust-version mycargo/*/Cargo.toml %{sources}/Cargo.toml | sort -u | tail -n 8


%build
%{?dtsenable}

export CARGO_HOME=$PWD/mycargo

cd %{sources}
%{__phpize}

cd ../NTS
%configure \
    --enable-skywalking_agent \
    --enable-kafka-reporter \
    --with-php-config=%{__phpconfig}
make %{?_smp_mflags}

%if %{with_zts}
cd ../ZTS
%configure \
    --enable-skywalking_agent \
    --enable-kafka-reporter \
    --with-php-config=%{__ztsphpconfig}
make %{?_smp_mflags}
%endif


%install
%{?dtsenable}

make -C NTS install INSTALL_ROOT=%{buildroot}

# install config file
install -D -m 644 %{ini_name} %{buildroot}%{php_inidir}/%{ini_name}

# Install XML package description
install -D -m 644 package.xml %{buildroot}%{pecl_xmldir}/%{name}.xml

%if %{with_zts}
make -C ZTS install INSTALL_ROOT=%{buildroot}

install -D -m 644 %{ini_name} %{buildroot}%{php_ztsinidir}/%{ini_name}
%endif

# Documentation
cd %{sources}
for i in $(grep 'role="doc"' ../package.xml | sed -e 's/^.*name="//;s/".*$//')
do install -Dpm 644 $i %{buildroot}%{pecl_docdir}/%{pecl_name}/$i
done
%{!?_licensedir:install -Dpm 644 LICENSE %{buildroot}%{pecl_docdir}/%{pecl_name}/LICENSE}


%if 0%{?fedora} < 24 && 0%{?rhel} < 8
# when pear installed alone, after us
%triggerin -- %{?scl_prefix}php-pear
if [ -x %{__pecl} ] ; then
    %{pecl_install} %{pecl_xmldir}/%{name}.xml >/dev/null || :
fi

# posttrans as pear can be installed after us
%posttrans
if [ -x %{__pecl} ] ; then
    %{pecl_install} %{pecl_xmldir}/%{name}.xml >/dev/null || :
fi

%postun
if [ $1 -eq 0 -a -x %{__pecl} ] ; then
    %{pecl_uninstall} %{pecl_name} >/dev/null || :
fi
%endif


%check
OPT="-n -d extension=curl.so -d extension=json.so"

# Minimal load test for NTS extension
%{__php} $OPT \
    --define extension=$PWD/NTS/modules/%{pecl_name}.so \
    --modules | grep '^%{pecl_name}$'

%if %{with_zts}
# Minimal load test for ZTS extension
%{__ztsphp} $OPT \
    --define extension=$PWD/ZTS/modules/%{pecl_name}.so \
    --modules | grep '^%{pecl_name}$'
%endif


%files
%{?_licensedir:%license %{sources}/LICENSE %{sources}/dist-material/licenses/LICENSE*}
%doc %{pecl_docdir}/%{pecl_name}
%{pecl_xmldir}/%{name}.xml

%config(noreplace) %{php_inidir}/%{ini_name}
%{php_extdir}/%{pecl_name}.so

%if %{with_zts}
%config(noreplace) %{php_ztsinidir}/%{ini_name}
%{php_ztsextdir}/%{pecl_name}.so
%endif


%changelog
* Thu Sep 28 2023 Remi Collet <remi@remirepo.net> - 0.7.0-1
- update to 0.7.0

* Fri Aug 25 2023 Remi Collet <remi@remirepo.net> - 0.6.0-1
- update to 0.6.0
- build out of sources tree

* Tue May  9 2023 Remi Collet <remi@remirepo.net> - 0.5.0-1
- update to 0.5.0

* Wed Mar 29 2023 Remi Collet <remi@remirepo.net> - 0.4.0-1
- update to 0.4.0

* Thu Feb  2 2023 Remi Collet <remi@remirepo.net> - 0.3.0-1
- update to 0.3.0

* Mon Nov 28 2022 Remi Collet <remi@remirepo.net> - 0.2.0-1
- update to 0.2.0

* Mon Oct 10 2022 Remi Collet <remi@remirepo.net> - 0.1.0-1
- initial package
- open https://github.com/apache/skywalking/pull/9754   Add PHP component
- open https://github.com/apache/skywalking/issues/9753 Bad role
- open https://github.com/apache/skywalking/issues/9755 8.2 support
- open https://github.com/apache/skywalking/issues/9756 GCC 12 support
- open https://github.com/apache/skywalking/issues/9757 rust 1.58
- open https://github.com/apache/skywalking/issues/9758 offline build
